pub struct CancellationTokenWrapper {
pub token: CancellationToken,
}Fields
token: CancellationTokenImplementations
Methods from Deref<Target = CancellationToken>
Sourcepub fn child_token(&self) -> CancellationToken
pub fn child_token(&self) -> CancellationToken
Creates a CancellationToken which will get cancelled whenever the
current token gets cancelled. Unlike a cloned CancellationToken,
cancelling a child token does not cancel the parent token.
If the current token is already cancelled, the child token will get returned in cancelled state.
Examples
use tokio::select;
use tokio_util::sync::CancellationToken;
#[tokio::main]
async fn main() {
let token = CancellationToken::new();
let child_token = token.child_token();
let join_handle = tokio::spawn(async move {
// Wait for either cancellation or a very long time
select! {
_ = child_token.cancelled() => {
// The token was cancelled
5
}
_ = tokio::time::sleep(std::time::Duration::from_secs(9999)) => {
99
}
}
});
tokio::spawn(async move {
tokio::time::sleep(std::time::Duration::from_millis(10)).await;
token.cancel();
});
assert_eq!(5, join_handle.await.unwrap());
}Sourcepub fn cancel(&self)
pub fn cancel(&self)
Cancel the CancellationToken and all child tokens which had been
derived from it.
This will wake up all tasks which are waiting for cancellation.
Be aware that cancellation is not an atomic operation. It is possible
for another thread running in parallel with a call to cancel to first
receive true from is_cancelled on one child node, and then receive
false from is_cancelled on another child node. However, once the
call to cancel returns, all child nodes have been fully cancelled.
Sourcepub fn is_cancelled(&self) -> bool
pub fn is_cancelled(&self) -> bool
Returns true if the CancellationToken is cancelled.
Sourcepub fn cancelled(&self) -> WaitForCancellationFuture<'_>
pub fn cancelled(&self) -> WaitForCancellationFuture<'_>
Sourcepub fn drop_guard_ref(&self) -> DropGuardRef<'_>
pub fn drop_guard_ref(&self) -> DropGuardRef<'_>
Creates a DropGuardRef for this token.
Returned guard will cancel this token (and all its children) on drop unless disarmed.
Sourcepub async fn run_until_cancelled<F>(
&self,
fut: F,
) -> Option<<F as Future>::Output>where
F: Future,
pub async fn run_until_cancelled<F>(
&self,
fut: F,
) -> Option<<F as Future>::Output>where
F: Future,
Runs a future to completion and returns its result wrapped inside of an Option
unless the CancellationToken is cancelled. In that case the function returns
None and the future gets dropped.
Fairness
Calling this on an already-cancelled token directly returns None.
For all subsequent polls, in case of concurrent completion and
cancellation, this is biased towards the future completion.
Cancellation safety
This method is only cancel safe if fut is cancel safe.